﻿/* Shaun Marquardt
 * TINFO 200
 * CS3: TicTacToe
 * ****************************************************
 * Change History
 * Date         Developer       Description
 * 2020-02-18   marqusa        File creation and initial implementation, 
 *                             Martyr2 (Coders Lexicon), Microsoft, Chuck Costarella L5Life
 * 2020-02-22   marqusa        Add functionality to clear the game board labels.
 *                             Implement Winner and Draw Game.
 *                             Completed program, add documentation.
 * 2020-02-23   marqusa        Add functionality to have CPU play as player 1 or player 2.
 *                             Moved winner stuff out of GameLbl_Click
 * 2020-02-24   marqusa        Fixed the form menu system for Play vs CPU checkboxes,
 *                             they were acting funky. Split PlayVsCPU(int turn) into new function.
 *                             Also updated the CPUTimer_Tick to do nothing
 *                             if the game is over.
 *                             
 * The Form1.cs class acts as the primary driver for the TicTacToe class
 * which actually plays the game proper. Most of my comments regarding how
 * the game actually works will be in the TicTacToe.cs file.
 * 
 * Recieves a delegate event from TicTacToe to update the label in the cell
 * that was clicked after the 3x3 2d array called 'gameBoard' in the
 * TicTacToe class gets updated
 * 
 * REFERENCES
 * https://www.coderslexicon.com/passing-data-between-forms-using-delegates-and-events/
 * https://docs.microsoft.com/en-us/dotnet/api/system.windows.forms.control.controlcollection.find?view=netframework-4.8
 * https://docs.microsoft.com/en-us/dotnet/api/system.eventargs?view=netframework-4.8
 * 
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace TicTacToe
{
    public partial class frmTicTacToe : Form
    {
        /* This is a program to play the classic game,
         * Tic-Tac-Toe. The form you see displayed (and this class)
         * is primarily a driver for the TicTacToe class
         * which actually plays the game behind the scenes.
         * 
         * Requirements:
         * 1) Class TicTacToe enables us to create a complete app to play
         * the game of Tic-Tac-Toe.
         * 2) This class contains a private 3x3 rectangular array of integers.
         * 3) The constructor initializes the empty board.
         * 4) Each move should be in an empty square.
         * 5) Determine win, lose, or draw.
         * 
         * Extra Credit:
         * The app is developed using a Windows Form application.
         * It is possible to play vs CPU with CPU as Player 1 or Player 2.
         * 
         * Extra Fun Stuff:
         * The game actively tracks the current player's turn.
         * I used an enum for both player turn and X and O.
         * I included a delegate and custom event args to pass
         * data from the TicTacToe class and update the correct label.
         * The game is really handled by basically everything in the
         * TicTacToe class with the form as a GUI frontend.
         */

        //Create a new instance of the TicTacToe class
        // and store it as a new object called 'game'.
        TicTacToe game;

        //Visual studio generated code.
        public frmTicTacToe()
        {
            InitializeComponent();
        }

        /* Update the label in a given row/column with X or O
         * according to what XorO enum value was placed using a delegate.
         * Ref: Martyr2, Coders Lexicon'
         * Preconditions: not yet
         * Inputs: object sender, TicTacToe.RowColumnEventArgs e
         * Outputs: void return
         * Postconditions: not yet
         */
        public void MessageReceived(object sender, EventArgs e)
        {
            //Ref: Microsoft, EventArgs
            TicTacToe.RowColumnEventArgs args = (TicTacToe.RowColumnEventArgs)e;

            //DEBUGGING:
            //MessageBox.Show($"row: {args.row} col: {args.col}");

            //Construct the label name we need.
            string lblName = $"lbl{args.row}{args.col}";

            //Get the label by constructed name.
            //ref: microsoft, Controls.Find
            Label theLabel = (Label)this.Controls.Find(lblName, true)[0];

            //Place an X or an O in the square.
            //Not playing against CPU is the easy case.
            //Player 1 is always X. Player 2 is O
            if (!Properties.Settings.Default.PlayVsCPU)
            {
                if (game.CurrentPlayerTurn == TicTacToe.PlayerTurn.Player1)
                    theLabel.Text = TicTacToe.XorO.x.ToString();
                else
                    theLabel.Text = TicTacToe.XorO.o.ToString();
            }
            else //We are playing against CPU.
            {
                //If it's the CPU turn, place the CPU move.
                if (game.CurrentPlayerTurn == TicTacToe.PlayerTurn.CPU)
                {
                    if (Properties.Settings.Default.CPUAsPlayer == 1)
                    {
                        theLabel.Text = TicTacToe.XorO.x.ToString();
                    }
                    else //CPU as player 2.
                    {
                        theLabel.Text = TicTacToe.XorO.o.ToString();
                    }
                }
                else //It's the player's turn
                {
                    //The CPU is player 1, place an O for human's move.
                    if (Properties.Settings.Default.CPUAsPlayer == 1)
                        theLabel.Text = TicTacToe.XorO.o.ToString();
                    else //The CPU is player 2, place an X for human's move.
                        theLabel.Text = TicTacToe.XorO.x.ToString();
                }
            }
        }

        /* Visual studio generated delegate
         * on form load. Instansiates a new TicTacToe class
         * and stores it into the game object.
         * Preconditions: not yet
         * Inputs: object sender, EventArgs e
         * Outputs: void return
         * Postconditions: not yet
         */
        private void frmTicTacToe_Load(object sender, EventArgs e)
        {
            //Pass the current TicTacToe form into the TicTacToe class
            //so the event handler delegate can be hooked in and we
            //can change the label in the tableGridView from the TicTacToe class.
            game = new TicTacToe(this);

            //If we are playing as CPU
            if (Properties.Settings.Default.PlayVsCPU)
            {
                playVsCPUToolStripMenuItem.Checked = true;

                //Check the CPU as player 1 tool strip menu item
                if (Properties.Settings.Default.CPUAsPlayer == 1)
                {
                    cPUAsPlayer1ToolStripMenuItem.Checked = true;
                }
                else //Check the CPU as player 2 tool strip menu item.
                {
                    cPUAsPlayer2ToolStripMenuItem.Checked = true;
                }
            }

            /* Start the timer to watch for CPU Turn.
             * This must be outside the if statement
             * as "Play vs CPU" can be checked (or unchecked) anytime
             * there is a "Game Over" status.
             */
            CPUTurnTimer.Start();
        }

        /* Starts a new game on Game > New Game click.
         * Visual Studio Generated Delegate.
         * Preconditions: not yet
         * Inputs: object sender, EventArgs e
         * Outputs: void return
         * Postconditions: not yet
         */
        private void newGameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            game.StartNewGame();
            ClearGameLabels();

            //Must set GameOver to true here
            //so if the user clicks the new game menu item
            //it is also possible to click the "Play vs CPU"
            //menu item/option.
            game.GameOver = true;
        }

        /* Clear all the labels on the board of X or O.
         * Preconditions: not yet
         * Inputs: no args
         * Outputs: void return
         * Postconditions: not yet
         */
        private void ClearGameLabels()
        {
            //Loop through each control docked in the
            //table main game layout panel that holds all
            //the labels for Xs and Os to go into.
            foreach (var tlpControl in tlpMainGame.Controls)
            {
                Label label = (Label)tlpControl;
                label.Text = string.Empty;
            }

            //Reset the player turn to Game Over.
            lblPlayerTurn.Text = "Player Turn: **GAME OVER **\nClick to Start";
        }

        /* Quit the game and exits the application.
         * Visual Studio generated delegate.
         * Preconditions: not yet
         * Inputs: object sender, EventArgs e
         * Outputs: void return
         * Postconditions: not yet
         */
        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Salutation
            MessageBox.Show("Thank you for playing! Good bye!");
            Environment.Exit(0);
        }

        /* This is the main delegate that is called when a player
         * clicks a position on the game board to play the game.
         * Visual Studio generated delegate.
         * Preconditions: not yet
         * Inputs: object sender, EventArgs e
         * Outputs: void return
         * Postconditions: not yet
         */
        private void GameBoardLbl_Click(object sender, EventArgs e)
        {
            //If the game is over, start a new game.
            if (game.GameOver)
            {
                game.StartNewGame();
                ClearGameLabels();

                //If the player is playing against CPU
                //and CPU is player 1, adjust the turn to CPU turn.
                if (Properties.Settings.Default.PlayVsCPU
                && Properties.Settings.Default.CPUAsPlayer == 1)
                    game.CurrentPlayerTurn = TicTacToe.PlayerTurn.CPU;
            }

            //If the player is playing against the CPU
            //and it is not their CurrentPlayerTurn, do nothing.
            if (Properties.Settings.Default.PlayVsCPU && game.CurrentPlayerTurn == TicTacToe.PlayerTurn.CPU)
                return;

            //Get the name of the label and parse it to find out our indexes.
            Label clickedGamePosition = (Label)sender;

            //Do not allow placing a move in a taken position.
            if (clickedGamePosition.Text != "")
                return;

            //Each label is named 'lblXX' where XX is the row and column
            //that was clicked. So first pick off 'lbl', to leave 'xx'
            string rowColumn = clickedGamePosition.Name.Remove(0, 3);

            //Now get the row and column.
            //ToString() must be called as rowColumn[x] returns type char.
            int row = int.Parse(rowColumn[0].ToString());
            int col = int.Parse(rowColumn[1].ToString());

            //Take the current player's CurrentPlayerTurn.
            game.TakePlayerTurn(game.CurrentPlayerTurn, row, col);

            //Swap to the next player's CurrentPlayerTurn.
            TicTacToe.XorO winningPlayer = game.TogglePlayerTurn(game.CurrentPlayerTurn);

            //Update the player turn label
            string playerStr = (game.CurrentPlayerTurn.ToString() == TicTacToe.PlayerTurn.CPU.ToString()) ? "CPU (THINKING...)" : game.CurrentPlayerTurn.ToString();
            lblPlayerTurn.Text = $"Player turn: {playerStr}";

            //Update winner
            //Check for a winner which will overwrite the turn label.
            UpdateWinner(winningPlayer);
        } //GameBoardLbl_Click

        /* Update the form label if there is a winner.
         * Preconditions: not yet
         * Inputs: XorO enum winningPlayer
         * Outputs: void return
         * Postconditions: not yet
         */
        private void UpdateWinner(TicTacToe.XorO winningPlayer)
        {
            //Check for a winner which will overwrite the turn label.
            if (game.GameOver) //There is a winner
            {
                string winningPlayerText = "**WINNER**: ";

                //Not playing against CPU, easy case.
                if (!Properties.Settings.Default.PlayVsCPU)
                {
                    if (winningPlayer == TicTacToe.XorO.x) //Player 1 wins.
                        winningPlayerText += "Player 1";
                    else if (winningPlayer == TicTacToe.XorO.o)
                        winningPlayerText += "Player 2"; //Player 2 wins.
                    else
                        winningPlayerText = "**DRAW GAME!**";
                }
                else // (PLAY VS CPU)
                {
                    //Player 1 CPU Wins
                    if (winningPlayer == TicTacToe.XorO.x && Properties.Settings.Default.CPUAsPlayer == 1)
                        winningPlayerText += "CPU";
                    else if (winningPlayer == TicTacToe.XorO.x) //Player 1 Wins
                        winningPlayerText += "Player 1";
                    else if (winningPlayer == TicTacToe.XorO.o && Properties.Settings.Default.CPUAsPlayer == 2) //player 2 cpu wins
                        winningPlayerText += "CPU";
                    else if (winningPlayer == TicTacToe.XorO.o) // player 2 wins
                        winningPlayerText += "Player 2";
                    else //Draw Game.
                        winningPlayerText = "**DRAW GAME!**";
                } //end else

                //Add Game Over text.
                winningPlayerText += "\n **GAME OVER** - Click to Start";

                //Update the player turn label with the winner.
                lblPlayerTurn.Text = winningPlayerText;
            } // if GameOver
        } //UpdateWinner

        /* Sets CPU as Player 1
         * Visual Studio generated delegate.
         * Preconditions: not yet
         * Inputs: object sender, EventArgs e
         * Outputs: void return
         * Postconditions: not yet
         */
        private void cPUAsPlayer1ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //If we are playing vs CPU and If the current box for
            //P1 CPU is checked, Clear PlayVsCPU options.
            if (Properties.Settings.Default.PlayVsCPU && cPUAsPlayer1ToolStripMenuItem.Checked == true)
            {
                SetPlayVsCPU(0);
            }
            else //We are not currently playing vs CPU. Start doing so with CPU as player 1.
            {
                SetPlayVsCPU(1);
            }
        }

        /* Sets CPU as Player 2
         * Visual Studio generated delegate.
         * Preconditions: not yet
         * Inputs: object sender, EventArgs e
         * Outputs: void return
         * Postconditions: not yet
         */
        private void cPUAsPlayer2ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //If we are playing vs CPU and If the current box for
            //P2 CPU is checked, Clear PlayVsCPU options.
            if (Properties.Settings.Default.PlayVsCPU && cPUAsPlayer2ToolStripMenuItem.Checked == true)
            {
                SetPlayVsCPU(0);
            }
            else //We are not currently playing vs CPU. Start doing so with CPU as player 2.
            {
                SetPlayVsCPU(2);
            }
        }

        /* Clears playing vs CPU if set.
         * Visual Studio generated delegate.
         * Preconditions: not yet
         * Inputs: object sender, EventArgs e
         * Outputs: void return
         * Postconditions: not yet
         */
        private void playVsCPUToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if(Properties.Settings.Default.PlayVsCPU)
                SetPlayVsCPU(0);
        }

        /* Set the correct menu checkboxes and application settings.
         * called from the menu tool strip delegate events.
         * Preconditions: not yet
         * Inputs: int cpuAsPlayer
         * Outputs: void return
         * Postconditions: not yet
         */
        private void SetPlayVsCPU(int cpuAsPlayer)
        {
            //If the game is not over, alert the player.
            if (!game.GameOver)
            {
                MessageBox.Show("A game is already underway, start a new one.");
                return;
            }

            //Set the correct menu checkboxes and application settings.
            Properties.Settings.Default.CPUAsPlayer = cpuAsPlayer;
            switch (cpuAsPlayer)
            {
                case 0:
                    Properties.Settings.Default.PlayVsCPU = false;
                    playVsCPUToolStripMenuItem.Checked = false;
                    cPUAsPlayer1ToolStripMenuItem.Checked = false;
                    cPUAsPlayer2ToolStripMenuItem.Checked = false;
                    break;
                case 1: //CPU as player 1
                    Properties.Settings.Default.PlayVsCPU = true;
                    playVsCPUToolStripMenuItem.Checked = true;
                    cPUAsPlayer1ToolStripMenuItem.Checked = true;
                    cPUAsPlayer2ToolStripMenuItem.Checked = false;
                    break;
                case 2: //CPU as player 2
                    Properties.Settings.Default.PlayVsCPU = true;
                    playVsCPUToolStripMenuItem.Checked = true;
                    cPUAsPlayer1ToolStripMenuItem.Checked = false;
                    cPUAsPlayer2ToolStripMenuItem.Checked = true;
                    break;
            }
            //Save the settings.
            Properties.Settings.Default.Save();
        } //SetPlayVsCPU

        /* Game timer delegate to watch for the CPU Turn.
         * Visual studio generated delegate.
         * Preconditions: not yet
         * Inputs: object sender, EventArgs e
         * Outputs: void return
         * Postconditions: not yet
         */
        private void CPUTurnTimer_Tick(object sender, EventArgs e)
        {
            //If the game is over, do not take a CPU turn.
            if (game.GameOver)
                return;

            TicTacToe.XorO winningPlayer;
            string playerStr = string.Empty;

            //Take the CPU turn.
            if (game.CurrentPlayerTurn == TicTacToe.PlayerTurn.CPU)
            {
                //Row and col can be 0,0 here as TakeCPUTurn will be called
                //which will randomly pick a row/col to play in.
                game.TakeCPUTurn();

                //Swap to the next player's turn
                winningPlayer = game.TogglePlayerTurn(game.CurrentPlayerTurn);

                //Update the player turn label after CPU turn.
                playerStr = (game.CurrentPlayerTurn.ToString() == TicTacToe.PlayerTurn.CPU.ToString()) ? "CPU (THINKING...)" : game.CurrentPlayerTurn.ToString();
                lblPlayerTurn.Text = $"Player turn: {playerStr}";

                UpdateWinner(winningPlayer);
            }
        } //CPUTurnTimer_Tick
    } // frmTicTacToe
}
