﻿/*
 * Change History
 * Date         Developer       Description
 * 2020-02-11   marqusa         File creation and initial implementation.
 *                                  -added student class to start with. Add user input and output to files.
 * 2020-02-13   marqusa         Continue with in-class work on the program, Chuck Costarella
 *                                  - Implemented and updated new input functionality.
 * 2020-02-18   marqusa         Continue with in-class work on the program, Chuck Costarella
 *                                  -Made GradStudent and Undergrad classes.
 *                                  -Implementing reading and writing of student types
 * 2020-02-20   marqusa         Continue with in-class work on the program, Chuck Costarella
 *                                  -Started coding for find, delete, and modify records
 * 2020-02-25   marqusa         Continue with in-class work on the program, Chuck Costarella
 *                                  -DeleteStudentRecord(), ListAllEmailAddresses()
 * 2020-02-26   marqusa         Fix the output error when writing to output file in wrong order in undergrad
 * 2020-02-27   marqusa         clean up adding new student.
 *                              Wrap deletion in a confirmation.
 *                              Move constants to top of StuDB
 *                              Add documentation.
 *                              Add shell to-do for inserting test student data into DB.
 *                              Add GetChoice with a maximum range above 9 as
 *                              GetSelectionFromUser only gets a single key, and does not 
 *                              support numbers above 9. Ref: Microsoft
 * 2020-02-28   marqusa         ModifyStudentRecord.
 * 2020-02-29   marqusa         Finish completely implementing ModifyStudentRecord/ModifyUnderGrad/ModifyGrad
 *                              Complete all listed TODO items other than adding more rules for email address.
 * REFERENCES
 * https://docs.microsoft.com/en-us/dotnet/csharp/programming-guide/strings/how-to-determine-whether-a-string-represents-a-numeric-value
 */
using System;
using System.Collections.Generic;
using System.IO;

namespace StudentDB
{
    internal class StuDB
    {
        //Constants.
        const string STUDENT_DB_DATA_FILE = "STUDENT_DB_DATA_FILE.txt";
        public const string UNDERGRAD = "UNDERGRAD";
        public const string GRADSTUDENT = "GRADSTUDENT";

        // the more or less persistent storage for students during runtime
        private List<Student> students = new List<Student>();

        /* Main entry point to the StuDB application.
         * Preconditions: not yet
         * Inputs: no args
         * Outputs: void return
         * Postconditions: not yet
         */
        public void GoDatabase()
        {
            //read the data in fron the input file
            ReadDataFromInputFile();

            // setup a loop that will repeatedly prompt the user for an action
            //CRUD - Create, Read, Update, Delete
            while (true)
            {
                // display a main menu to the user
                DisplayMainMenu();

                // CRUD - capture the user selection
                ConsoleKeyInfo selection = GetSelectionFromUser();

                //Add space after reading the key.
                Console.WriteLine();
                switch (selection.KeyChar)
                {
                    case 'I':
                    case 'i':
                        InsertTestStudentsIntoDB();
                        break;
                    case 'L':
                    case 'l':
                        ListAllEmailAddresses();
                        break;
                    case 'M':
                    case 'm':
                        ModifyStudentRecord();
                        break;
                    case 'D':
                    case 'd':
                        DeleteStudentRecord();
                        break;
                    case 'F':
                    case 'f':
                        Console.Write("Enter email ID to search for: ");
                        string email = Console.ReadLine();
                        Student searchStudent = FindStudentRecord(email);
                        if (searchStudent != null)
                        {
                            Console.WriteLine(searchStudent);
                        }
                        else
                        {
                            Console.WriteLine($"{email} not currently in the database.");
                        }
                        break;
                    case 'N':
                    case 'n':
                        System.Diagnostics.Process.Start("notepad");
                        break;
                    case 'A':
                    case 'a':
                        AddStudentRecord();
                        break;
                    case 'P':
                    case 'p':
                        PrintAllRecords();
                        break;
                    case 'Q':
                    case 'q':
                    case 'X':
                    case 'x':
                        ExitTheDatabaseApp();
                        break;
                    case 'S':
                    case 's':
                        SaveDataToOutputFile();
                        break;
                    default:
                        Console.WriteLine($"... sorry, '{selection.KeyChar}' isn't a valid choice.");
                        break;
                }
            }

        }

        /* List all email addresses currently in the Student list
         * Preconditions: not yet
         * Inputs: no args
         * Outputs: void return
         * Postconditions: not yet
         */
        private void ListAllEmailAddresses()
        {
            Console.WriteLine("*************************");
            for (int i = 0; i < students.Count; i++)
            {
                Console.WriteLine($"{i + 1}: {students[i].EmailAddress}");
            }
            Console.WriteLine("END of email list");
        }

        /* Deletes a student record out of the in-memory student list
         * after asking "Are you sure you want to delete?"
         * The confirmation takes Y or y for yes, and 
         * anything else will not delete the student.
         * Preconditions: not yet
         * Inputs: no args
         * Outputs: void return
         * Postconditions: not yet
         */
        private void DeleteStudentRecord()
        {
            //show the user a list of possible deletions in concise format
            ListAllEmailAddresses();

            Console.Write("Your selection: ");
            int recordToDelete = GetChoice(Console.ReadLine(), students.Count);

            //adjust the record to delete to correctly index.
            //GetChoice returns -1 if choice is not valid, which is checked in the below ifcheck.
            recordToDelete -= 1;

            //Insert a space to pretty up the formatting.
            Console.WriteLine();

            // Check to see if recordToDelete is between 0 and <students.Count
            if (recordToDelete < 0 || recordToDelete > students.Count - 1)
            {
                Console.WriteLine("Sorry, you did not make a valid choice.");
                return;
            }

            ConsoleKeyInfo confirmDelete = GetSelectionFromUser($"Are you sure you want to delete {students[recordToDelete].EmailAddress} [y/n]? ");
            Console.WriteLine();

            switch (confirmDelete.KeyChar)
            {
                case 'Y':
                case 'y':
                    students.RemoveAt(recordToDelete);

                    //Say something nice and make the user feel all warm and fuzzy
                    Console.WriteLine("After delete, now the list is:");
                    ListAllEmailAddresses();
                    break;
                default:
                    Console.WriteLine("Record NOT deleted.");
                    break;
            }
        }

        /* Modify a student record.
         * Preconditions: not yet
         * Inputs: no args
         * Outputs: void return
         * Postconditions: not yet
         * 
         * NOTE: There is no requirement to be able to modify a student's subtype,
         * i.e, an Undergrad need not be editable to become a GradStudent, or vice versa
         */
        private void ModifyStudentRecord()
        {
            ListAllEmailAddresses();
            Console.Write("Your selection: ");
            int choice = GetChoice(Console.ReadLine(), students.Count);

            //Decrement choice to properly index into the student list.
            choice -= 1;

            //GetChoice returns -1 if choice is not valid.
            //So any negative choice "index" is not valid.
            if (choice < 0)
            {
                Console.WriteLine("... sorry, that's an invalid choice.");
                return;
            }

            //The record is found. Show the record to the user.
            var studentToModify = students[choice];

            //If this is a grad student, cast studentToModify
            //into GradStudent type.
            if (studentToModify is GradStudent)
                studentToModify = (GradStudent)studentToModify;
            else
                studentToModify = (Undergrad)studentToModify;

            //Display a sub-menu for edit
            DisplayEditMenu(studentToModify);

            // when done editing-
            // display new version of record
            Console.WriteLine("\nHere is the new record of that student after modification:");
            Console.Write(studentToModify);
        }

        /* Modify student / Edit submenu
         * Preconditions: not yet
         * Inputs: Student
         * Outputs: void return
         * Postconditions: not yet
         */
        private void DisplayEditMenu(Student studentToModify)
        {
            //Must be up here to check on While, below.
            ConsoleKeyInfo selection;

            //Shows the edit menu until the choice is Q.
            do
            {
                Console.WriteLine(@"
********************************
******* Edit Student Menu ******
********************************
");
                //Display something to the user indicating what type of student this is.
                if (studentToModify is GradStudent)
                    Console.Write("This is a Grad Student.");
                else
                    Console.Write("This is an Undergrad Student.");

                Console.Write(@"
What data for this student would you like to change?
    [F]irst name: " + studentToModify.FirstName +
    "\n    [L]ast name: " + studentToModify.LastName +
    "\n    [G]PA: " + studentToModify.GradePtAvg +
    "\n    [E]mail: " + studentToModify.EmailAddress
    );
                if (studentToModify is GradStudent)
                    ShowGradStudentEditMenu((GradStudent)studentToModify);
                else
                    ShowUndergradEditMenu((Undergrad)studentToModify);

                Console.WriteLine(@"    [Q]uit editing this student.");

                // capture the user selection
                selection = GetSelectionFromUser();

                //Add space after reading the key.
                Console.WriteLine();

                switch (selection.KeyChar)
                {
                    case 'F': //FirstName
                    case 'f':
                        Console.Write("Enter new first name: ");
                        string newFirstName = Console.ReadLine();
                        studentToModify.FirstName = newFirstName;
                        break;
                    case 'L': //LastName
                    case 'l':
                        Console.Write("Enter new last name: ");
                        string newLastName = Console.ReadLine();
                        studentToModify.LastName = newLastName;
                        break;
                    case 'G': //GPA
                    case 'g':
                        Console.Write("Enter new GPA between 0 and 4.0 inclusive: ");
                        double newGPA = double.Parse(Console.ReadLine());
                        studentToModify.GradePtAvg = newGPA;
                        break;
                    case 'E': //Email Address
                    case 'e':
                        Console.Write("Enter the new valid email address with @ symbol: ");
                        string newEmailAddy = Console.ReadLine();
                        //Check to see if another student has the desired email address.
                        if (FindStudentRecord(newEmailAddy) == null)
                            studentToModify.EmailAddress = newEmailAddy;
                        else
                            Console.WriteLine("Sorry, another student has that email. Try again.");
                        break;
                    //Take care of modifying gradstudents.
                    case 'T': //Tuition credit
                    case 't':
                        if (studentToModify is GradStudent)
                            ModifyGradStudent((GradStudent)studentToModify, 't');
                        break;
                    case 'A': //Faculty [A]dvisor
                    case 'a':
                        if (studentToModify is GradStudent)
                            ModifyGradStudent((GradStudent)studentToModify, 'a');
                        break;
                    //Take care of modifying undergrad students.
                    case 'Y': //YearRank
                    case 'y':
                        if (studentToModify is Undergrad)
                            ModifyUndergrad((Undergrad)studentToModify, 'y');
                        break;
                    case 'D': //DegreeMajor
                    case 'd':
                        if (studentToModify is Undergrad)
                            ModifyUndergrad((Undergrad)studentToModify, 'd');
                        break;
                    case 'Q': //Done editing student.
                    case 'q':
                        break;
                    default: //Invalid selection
                        Console.WriteLine("...Sorry, that was an invalid choice.");
                        break;
                } //end switch
            } while (selection.KeyChar.ToString().ToUpper() != "Q"); //Check if the user is done editing.
        } // end DisplayEditMenu

        /* Modify student
         * This function is for Grad Students.
         * Preconditions: not yet
         * Inputs: GradStudent, char choice as 'T' or 'A'
         * Outputs: void return
         * Postconditions: not yet
         */
        private void ModifyGradStudent(GradStudent studentToModify, char choice)
        {
            switch (choice)
            {
                case 'T': //Modify the TuitionCredit
                case 't':
                    Console.Write("Enter the amount of the tuition credit without the dollar sign: ");
                    decimal newCredit = decimal.Parse(Console.ReadLine());
                    studentToModify.TuitionCredit = newCredit;
                    break;
                case 'A': //Modify the faculty advisor
                case 'a':
                    Console.Write("Enter the name of the faculty advisor: ");
                    studentToModify.FacultyAdvisor = Console.ReadLine();
                    break;
                default: //We shouldn't get here, but if we do, display an error to the user.
                    Console.WriteLine("**BUG**: ModifyGradStudent got called with an invalid selection. Please contact developer.");
                    break;
            }
        }

        /* Modify student
         * This function is for Undergrad Students.
         * Preconditions: not yet
         * Inputs: Undergrad, choice as 'Y' or 'D'
         * Outputs: void return
         * Postconditions: not yet
         */
        private void ModifyUndergrad(Undergrad studentToModify, char choice)
        {
            switch (choice)
            {
                //Modify the YearRank of the undergrad student.
                case 'Y':
                case 'y':
                    YearRank rank;
                    Console.Write("Enter the year in school as Freshman, Sophmore, Junior, or Senior: ");
                    //If it's a valid rank, save to the student.
                    if (Enum.TryParse(Console.ReadLine(), out rank))
                        studentToModify.Rank = rank;
                    else
                        Console.WriteLine("Sorry, that was invalid year, please try again.");
                    break;
                case 'D':
                case 'd':
                    Console.Write("Enter the student's Degree Major: ");
                    studentToModify.DegreeMajor = Console.ReadLine();
                    break;
                default: //We shouldn't get here, but if we do, display an error to the user.
                    Console.WriteLine("**BUG**: ModifyUndergrad got called with an invalid selection. Please contact developer.");
                    break;
            }
        }

        /* Modify student / Edit submenu
         * This function is for Undergrad Students.
         * Preconditions: not yet
         * Inputs: Undergrad
         * Outputs: void return
         * Postconditions: not yet
         */
        private void ShowUndergradEditMenu(Undergrad studentToModify)
        {
            Console.WriteLine(@"
    [Y]ear Rank: " + studentToModify.Rank +
"\n    [D]egree Major: " + studentToModify.DegreeMajor
);
        }

        /* Modify student / Edit submenu
         * This function is for Grad Students.
         * Preconditions: not yet
         * Inputs: GradStudent
         * Outputs: void return
         * Postconditions: not yet
         */
        private void ShowGradStudentEditMenu(GradStudent studentToModify)
        {
            Console.WriteLine(@"
    [T]uition Credit: " + studentToModify.TuitionCredit +
"\n    [A]dvisor (Faculty): " + studentToModify.FacultyAdvisor
);
        }

        /* Gets user integer choice between 1 and maxChoiceNumber
         * Preconditions: not yet
         * Inputs: string choice, maxChoiceNumber
         * Outputs: integer or -1 if not a valid choice.
         * Postconditions: not yet
         */
        private int GetChoice(string userChoice, int maxChoiceNumber)
        {
            int choice;

            //Check if choice is an integer (as a string).
            //ref: microsoft
            if (int.TryParse(userChoice, out choice))
            {
                //If the choice is not between 0 and range, return -1.
                if (choice < 0 || choice > maxChoiceNumber)
                    return -1;
                else
                    return choice;
            }
            else //If the choice is not an integer, return -1.
                return -1;
        }

        /* searches the list collection for a student and either returns
         * a reference to the found student or null if not found.
         * Preconditions: not yet
         * Inputs: string email address
         * Outputs: Student or null
         * Postconditions: not yet
         */
        private Student FindStudentRecord(string email)
        {
            //Iterate through each student type in the student list.
            foreach (var student in students)
            {
                //If the currently focused student's email address
                //is equal to the passed in email argument,
                //return the student.
                if (student.EmailAddress == email)
                {
                    return student;
                }
            }

            //Student not found, return null.
            return null;
        }

        /* Add a student record.
         * Preconditions: not yet
         * Inputs: no args
         * Outputs: void return
         * Postconditions: not yet
         */
        private void AddStudentRecord()
        {
            //First prompt for the email address as that is the key
            //discriminator between students in the database --
            //no two students may have the same email address.
            Console.Write("Desired email address: ");
            string email = Console.ReadLine();
            Student studentToAdd = FindStudentRecord(email);

            // if the key (email address) does not already exist - add the student
            if (studentToAdd == null)
            {
                Console.Write("First Name:");
                string first = Console.ReadLine();
                Console.Write("Last Name: ");
                string last = Console.ReadLine();
                Console.Write("GPA: ");
                double gpa = double.Parse(Console.ReadLine());

                //Prompt user for graduate student or not
                //to add the correct type of student to the list.
                switch (GetSelectionFromUser("Graduate student[y/n]? ").KeyChar)
                {
                    case 'Y':
                    case 'y':
                        //Grad student
                        Console.Write("\nTotal amount of tuition credit: ");
                        decimal credit = decimal.Parse(Console.ReadLine());
                        Console.Write("Name of faculty advisor: ");
                        string advisor = Console.ReadLine();

                        //now make a GradStudent and add to the list
                        students.Add(new GradStudent(first, last, gpa, email, credit, advisor));
                        break;
                    default:
                        //undergrad student
                        //Tell the user how to enter the year as it is an enum.
                        Console.Write("\nEnter the year in school as Freshman, Sophmore, Junior, or Senior: ");
                        YearRank rank;
                        Enum.TryParse<YearRank>(Console.ReadLine(), out rank);
                        Console.Write("What is the student's major: ");
                        string degree = Console.ReadLine();

                        // make a new undergrad stu with the gathered info and insert in the list
                        students.Add(new Undergrad(first, last, gpa, email, rank, degree));
                        break;
                }
            }
            else //A student with that email address was found
            {
                //Another student with that email address was found.
                Console.WriteLine("Sorry, another student with that email address is already in the database.");
            }
        }

        /* Prints all the student records in the list.
         * Preconditions: not yet
         * Inputs: no args
         * Outputs: void return
         * Postconditions: not yet
         */
        private void PrintAllRecords()
        {
            //For each type of student in the list,
            foreach (var student in students)
            {
                //call the type of student object's ToString() overridden method.
                Console.WriteLine(student);
            }
        }

        /* Save data to the output file.
         * Preconditions: not yet
         * Inputs: no args
         * Outputs: void return
         * Postconditions: not yet
         */
        private void SaveDataToOutputFile()
        {
            // create a file object to attach to the actual file on disk
            StreamWriter outfile = new StreamWriter(STUDENT_DB_DATA_FILE);

            //Debugging
            //StreamWriter outfile = new StreamWriter("OUTPUT.txt");

            // use it - write the contents of the students collection to the file
            for (int i = 0; i < students.Count; i++)
            {
                //There is no CRLF at the end of a student from this code
                outfile.Write(students[i].ToStringForOutputFile());
            }

            //release the resource
            outfile.Close();
        }

        /* Exits the application.
         * this method also saves all the current changes that have been made to the list.
         * Preconditions: not yet
         * Inputs: no args
         * Outputs: void return
         * Postconditions: not yet
         */
        private void ExitTheDatabaseApp()
        {
            //salutation
            Console.WriteLine("Thank you for using the Student Database Application.\n" +
                "Your work has been saved. Good bye!");

            //save the data.
            SaveDataToOutputFile();

            // zero typically means no errors- normal exit
            Environment.Exit(0);
        }

        /* Get the user selected operation character
         * Preconditions: not yet
         * Inputs: string prompt (default "User Selection: ")
         * Outputs: ConsoleKeyInfo (single key)
         * Postconditions: not yet
         */
        private ConsoleKeyInfo GetSelectionFromUser(string prompt = "User selection: ")
        {
            Console.Write(prompt);
            return Console.ReadKey();
        }

        /* Display the main menu of the application.
         * This is the main user interface of the program.
         * Preconditions: not yet
         * Inputs: no args
         * Outputs: void return
         * Postconditions: not yet
         */
        private void DisplayMainMenu()
        {
            Console.WriteLine(@"
********************************
***** Student Database App *****
********************************
    [P]rint all records
    [F]ind a student record
    [L]ist all email addresses
    [A]dd a new student record
    [D]elete a student record
    [M]odify a student record
    [N]otes utility
    [I]nsert test driver student data into DB
    [S]ave the records and continue
    [Q]uit database app (includes save)
");
        }

        /* Read in the input file data.
         * make student objects from the read in data
         * and store them in a semi-persistent data structure.
         * Preconditions: not yet
         * Inputs: no args
         * Outputs: void return
         * Postconditions: not yet
         */
        private void ReadDataFromInputFile()
        {
            StreamReader infile = null;

            //The student database file exists, read it.
            if (File.Exists(STUDENT_DB_DATA_FILE))
            {
                infile = new StreamReader(STUDENT_DB_DATA_FILE);
            }
            else //The file does not yet exist, do not load data.
            {
                return;
            }

            string studentType = string.Empty;

            //Boilerplate code to read for a file.
            //Read the file line by line, until the end of the file.
            //Once infile.readline() reaches EOF, str becomes NULL.
            while ((studentType = infile.ReadLine()) != null)
            {
                string firstName = infile.ReadLine();
                string lastName = infile.ReadLine();
                double grades = double.Parse(infile.ReadLine());
                string email = infile.ReadLine();

                if (studentType == UNDERGRAD)
                {
                    //reading into an enum type.
                    //chapter 7, book
                    YearRank rank;
                    Enum.TryParse<YearRank>(infile.ReadLine(), out rank);

                    string degree = infile.ReadLine();
                    students.Add(new Undergrad(firstName, lastName, grades,
                                                email, rank, degree));

                }
                else if (studentType == GRADSTUDENT)
                {
                    decimal credit = decimal.Parse(infile.ReadLine());
                    string advisor = infile.ReadLine();

                    //The reason we can put a grad student in a list of students
                    //is because a GradStudent IS A KIND OF student.
                    students.Add(new GradStudent(firstName, lastName, grades,
                                                email, credit, advisor));
                }
                else
                {
                    Console.WriteLine($"ERROR: Unsupported student type: {studentType}");
                }

                //Now that we have variables for a student,
                //Create a student object and add it to our list.
                //new Student is an anonymous object - it doesn't have a name.

                //now that all data for an obj has been read in - make the object and insert in the list
                //No longer need this code because of the above switch statement.
                //students.Add(new Student(firstName, lastName, grades, email));
            }

            //Release the resource.
            infile.Close();
        }

        /* Inserts test students into the student list
         * if those test students are not already present.
         * Preconditions: not yet
         * Inputs: no args
         * Outputs: void return
         * Postconditions: not yet
         */
        private void InsertTestStudentsIntoDB()
        {
            List<Student> testStudents = new List<Student>();

            //The test driver inserts a list of 8 students
            //into the Student list, if they are not already present.
            //First setup the Grad Students -- Pro Wrestlers.
            GradStudent gs1 = new GradStudent("Hulk", "Hogan", 4.0, "hhogan@uw.edu", (decimal)45000.99, "Dr. Hulkamaiac");
            testStudents.Add(gs1);

            GradStudent gs2 = new GradStudent("Randy", "Savage", 2.8, "rsavage@uw.edu", (decimal)42350.99, "Dr. Hurtzalot");
            testStudents.Add(gs2);

            GradStudent gs3 = new GradStudent("Rick", "Flair", 1.2, "rflair@uw.edu", (decimal)23000.99, "Dr. Dirtyplayer");
            testStudents.Add(gs3);

            GradStudent gs4 = new GradStudent("Bret", "Hart", 1.2, "bhart@uw.edu", (decimal)32975.98, "Dr. Hart");
            testStudents.Add(gs4);

            //Now setup the Undergrad Students -- Football players with hilarious degrees.
            Undergrad ug1 = new Undergrad("Tom", "Brady", 3.4, "tbrady@uw.edu", YearRank.Freshman, "BS-Deflategate");
            testStudents.Add(ug1);

            Undergrad ug2 = new Undergrad("Michael", "Vick", 2.5, "mvick@uw.edu", YearRank.Sophmore, "BS-Dogfighting");
            testStudents.Add(ug2);

            Undergrad ug3 = new Undergrad("Russel", "Wilson", 1.2, "rwilson@uw.edu", YearRank.Junior, "BS-PaidTooMuch");
            testStudents.Add(ug3);

            Undergrad ug4 = new Undergrad("Joe", "Montana", 3.9, "jmontana@uw.edu", YearRank.Senior, "BS-BeingTheMan");
            testStudents.Add(ug4);

            //Now for each of our test students in the list,
            //If the test student is NOT already in the database, add them.
            foreach (var student in testStudents)
            {
                if (FindStudentRecord(student.EmailAddress) == null)
                    students.Add(student);
            }

            Console.WriteLine("Test student data has been added if it was not present.");
            return;
        }
    }
}