﻿/* This is the Student class. Undergrad and GradStudent "is a type of" student.
 * See StuDB for full change history.
 * 
 * Change History
 * Date         Developer       Description
 * 2020-02-13   marqusa         In-class work on the program, Chuck Costarella
 * through 25
 */
namespace StudentDB
{
    internal class Student 
    {
        // auto implemented props
        public string FirstName { get; set; }

        public string LastName { get; set; }

        // instance variable
        private double gradePtAvg;
        private string emailAddress;

        // 1 - allocate storage for the object
        // 2 - init all vars to reasonable defaults
        //If first is not called with a variable, it has a default.
        //default values have to be piled up on the right side of the constructor
        public Student(string first,
                       string last = "Last name",
                       double grades = 0.7,
                       string email = "newstudent@uw.edu")
        {
            FirstName = first;
            LastName = last;
            GradePtAvg = grades;
            EmailAddress = email;
        }

        //Public variable to set the private emailAddress instance variable.
        public string EmailAddress
        {
            get
            {
                return emailAddress;
            }
            set
            {
                // TODO: Add more "rules" to what exactly an email can be.
                // -- ABANDONED --
                if(value.Contains("@"))
                {
                    emailAddress = value;
                }
            }
        }

        //Public variable to set the private gradePtAvg instance variable.
        public double GradePtAvg
        {
            get
            {
                return gradePtAvg;
            }
            set
            {
                // check if GPA is in valid range
                if (0.0 <= value && value <= 4.0)
                {
                    gradePtAvg = value;
                }
            }
        }

        // specifically formatted for the output/input file
        // no labels on any of the data
        // The virtual keyword allows for the method to be overriden.
        public virtual string ToStringForOutputFile()
        {
            // mkae an empty string
            string str = string.Empty;

            // build up the string with data we want to print
            // note this finishes a student output with a CRLF
            str += $"{FirstName}\n";
            str += $"{LastName}\n";
            str += $"{GradePtAvg}\n";
            str += $"{EmailAddress}\n";

            // return the string
            return str;
        }

        //Data to go to the user
        //Overrides the ToString method found in "Object".
        public override string ToString()
        {
            // mkae an empty string
            string str = string.Empty;

            // build up the string with data that you want to print from the object
            str += "***************************\n";
            str += $"First: {FirstName}\n";
            str += $" Last: {LastName}\n";
            str += $"  GPA: {GradePtAvg:0.0}\n";
            str += $"Email: {EmailAddress}\n";

            // return the string
            return str;
        }
    }
}