﻿/* This is the Undergrad class which inherits from Student class.
 * See StuDB for full change history.
 * It adds YearRank enum type and MajorDegree data to a student.
 * 
 * Change History
 * Date         Developer       Description
 * 2020-02-13   marqusa         In-class work on the program, Chuck Costarella
 * through 25
 * 2020-02-26   marqusa         Fix the output error when writing to output file in wrong order in undergrad
 * 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace StudentDB
{
    public enum YearRank
    {
        Freshman = 1,
        Sophmore = 2,
        Junior = 3,
        Senior = 4
    }

    // The way we indicate inheritance is using a colon, then indicate the base class.
    class Undergrad : Student
    {
        public YearRank Rank { get; set; }

        public string DegreeMajor { get; set; }

        //Must have at least the same constructor as the parent.
        //use :base() to call the parent constructor and supply the arguments
        //that the parent class needs.
        public Undergrad(string first, string last, double gpa, string email, YearRank rank, string degree)
            : base(first, last, gpa, email)
        {
            Rank = rank;
            DegreeMajor = degree;
        }

        // conversion to a lamda expression - mechanism of functional programming.
        // lamda operator is "=>" and reads "goes to"
        // C# calls this an "expression-bodied method"
        // This is only used for one-liners or very simple things.
        public override string ToString() => base.ToString() + $"Year: {Rank}\nDegree: {DegreeMajor}\n";

        public override string ToStringForOutputFile()
        {
            string str = StuDB.UNDERGRAD + "\n" + base.ToStringForOutputFile();

            // append the "local information"
            str += $"{Rank}\n";
            str += $"{DegreeMajor}\n";

            return str;
        }
    }
}
